#include <Plasma/FrameSvg>
#include <Plasma/Theme>
#include <Plasma/Wallpaper>
#include <KApplication>
#include <qfile.h>
#include <qtextstream.h>
#include <kstandarddirs.h>

#include <Plasma/Plasma>
#include <QPainter>

#include <QTime>
#include <QImage>
#include <QString>
#include <QPainter>
#include <QPixmap>
#include <QDesktopWidget>

#include <KIcon>

using namespace std;


#include <kcmdlineargs.h>
#include <klocale.h>
#include <kaboutdata.h>

const int iconSize = 24, textHeight = 15;
const int barW = iconSize * 7, barH = iconSize;
const int inputW = 200, inputH = textHeight * 2 + 2;

void saveThemeFile(QString themeFileName,QString imagePath)
{
	QFile f(themeFileName);

	if (f.open(QFile::WriteOnly | QFile::Truncate)) {
		QTextStream out(&f);
		Plasma::FrameSvg svg;
		svg.setImagePath(imagePath);
		svg.setEnabledBorders(Plasma::FrameSvg::AllBorders);
		svg.resizeFrame(QSize(200, 200));

        Plasma::Theme *theme = Plasma::Theme::defaultTheme();
        QColor color = theme->color(Plasma::Theme::TextColor);

		qreal left = 0;
		qreal top = 0;
		qreal right = 0;
		qreal bottom = 0;

		svg.getMargins(left,top,right,bottom);

        int logox = left, logoy= top; // 1, 2
        int imx = logox + iconSize, imy = top; //3, 4
        int simpx = imx + iconSize, simpy = top; //5, 6
        int fullx = simpx + iconSize, fully = top; //7, 8
        int puncx = fullx + iconSize, puncy = top; // 9, 10
        int vkx = puncx + iconSize, vky = top; // 11, 12
        int legendx = vkx + iconSize, legendy = top; //13, 14
        int resizepos = left, resizew = inputW;//15 16
        int inputp = top + textHeight, outputp = inputp + textHeight + 2;// 17 18
        int lleft = left, lright = right + textHeight; // 19 20
        int backx = left + inputW - 16, backy = top; // 21, 22
        int forwx = left + inputW - 32, forwy = top; // 23, 24
        int r,g,b;
        color.getRgb(&r, &g, &b);

		out<<QString("\
[SkinInfo]\n\
Name=Plasma\n\
Version=0.1\n\
Author=CSSlayer\n\
Desc=Based on plasma\n\
\n\
[SkinFont]\n\
FontSize=15\n\
FontEn=\n\
FontZh=\n\
InputCharColor=%25 %26 %27\n\
OutputCharColor=%25 %26 %27\n\
CharNoColor=%25 %26 %27\n\
FirstCharColor=%25 %26 %27\n\
\n\
[SkinMainBar]\n\
BackImg=bar.png\n\
Logo=fcitx.png %1 %2\n\
ZhPunc=fcitx-full-punct.png %9 %10\n\
EnPunc=fcitx-half-punct.png %9 %10\n\
Chs=fcitx-simp.png %5 %6\n\
Cht=fcitx-trad.png %5 %6\n\
HalfCorner=fcitx-half-letter.png %7 %8\n\
FullCorner=fcitx-full-letter.png %7 %8\n\
Unlock=\n\
Lock=\n\
Legend=fcitx-legend.png %13 %14\n\
NoLegend=fcitx-nolegend.png %13 %14\n\
NoVK=fcitx-vkoff.png %11 %12\n\
VK=fcitx-vkon.png %11 %12\n\
Eng=fcitx-eng.png %3 %4\n\
Chn=fcitx-chn.png %3 %4\n\
\n\
[SkinInputBar]\n\
BackImg=input.png\n\
Resize=Resize\n\
ResizePos=%15\n\
ResizeWidth=%16\n\
InputPos=%17\n\
OutputPos=%18\n\
LayoutLeft=%19\n\
LayoutRight=%20\n\
CursorColor=99 99 99\n\
BackArrow=next.png %21 %22\n\
ForwardArrow=prev.png %23 %24\n\
\n\
[SkinTrayIcon]\n\
Active=active.png\n\
Inactive=inactive.png\n\
").arg(logox).arg(logoy).arg(imx).arg(imy).arg(simpx).arg(simpy).arg(fullx).arg(fully).arg(puncx).arg(puncy).arg(vkx).arg(vky)
  .arg(legendx).arg(legendy).arg(resizepos).arg(resizew).arg(inputp).arg(outputp).arg(lleft).arg(lright).arg(backx).arg(backy).arg(forwx).arg(forwy).arg(r).arg(g).arg(b);
		f.close();
	}

}


void saveThemeElement(QString filename, QPixmap svg, int a)
{
    int w = svg.width();
    int h = svg.height();

	QPixmap pixmap(w,h);
	pixmap.fill(Qt::transparent);
	QPainter p(&pixmap);
	p.drawPixmap(pixmap.rect(),svg,QRectF(0,0,w,h));
	p.end();

	if(a >= 0 && a <= 255) {
		QPixmap alpha(w,h);
		QPainter p1(&alpha);
		p1.fillRect(QRectF(0,0,w,h),QColor(a,a,a));
		p1.end();
		pixmap.setAlphaChannel(alpha);
	}

	pixmap.save(filename,"PNG");
}


void saveBoxPixmaps(QString themePath,QString imagePath, int alpha)
{
    QSize s(200,200);
	Plasma::FrameSvg svg;
	svg.setImagePath(imagePath);
	svg.setEnabledBorders(Plasma::FrameSvg::AllBorders);
	svg.resizeFrame(s);

	qreal left = 0;
	qreal top = 0;
	qreal right = 0;
	qreal bottom = 0;

	svg.getMargins(left,top,right,bottom);

    QSize barSize(left + right + barW, top + bottom + barH);
    QSize inputSize(left + right + inputW, top + bottom + inputH);

    svg.resizeFrame(barSize);
	saveThemeElement(themePath+"bar.png",svg.framePixmap(),alpha);

    svg.resizeFrame(inputSize);
	saveThemeElement(themePath+"input.png",svg.framePixmap(),alpha);
}


static KCmdLineOptions options ()
{
	KCmdLineOptions options;
	options.add("a");
	options.add("alpha <alpha>",	ki18n("Set alpha of border (0-255)"));
	options.add("p");
	options.add("imagepath <imagepath>",		ki18n("Set image path of Plasma svg. "
				"'dialogs/background' and 'opaque/dialogs/backgrounds' are recommended. "
				"default is 'opaque/dialogs/backgrounds'."));
	return options;
}


int main( int argc, char **argv )
{

	KAboutData aboutData( "FcitxThemeGenerator", 
			"FcitxThemeGenerator", 
			ki18n("FcitxThemeGenerator"), 
			"0.2", 
			ki18n("A theme generator for Fcitx"),
			KAboutData::License_GPL, ki18n("Copyright (c) 2009 Weng Xuetian") );

	KCmdLineArgs::init( argc, argv, &aboutData );
	KCmdLineArgs::addCmdLineOptions(options());

	KApplication a;

	QString myFcitxHome= QString(getenv("HOME"))+"/.config/fcitx/skin/plasma/";

	if (!QFile(myFcitxHome).exists())  KStandardDirs::makeDir(myFcitxHome,0755);

	QString imagePath="opaque/dialogs/background";

	int alpha = -1;

	KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
	if (args->isSet("alpha")) alpha = args->getOption("alpha").toInt();
	if (args->isSet("imagepath")) imagePath = args->getOption("imagepath");

	args->clear();

	saveBoxPixmaps(myFcitxHome, imagePath, alpha);
	saveThemeFile(myFcitxHome + "fcitx_skin.conf", imagePath);

    QString cmd = "tar xvf png.tar.gz -C " + myFcitxHome;
    system(cmd.toStdString().data());

	return 0;
}

